/*
 * Copyright [2011] [Risk Focus Inc]
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.riskfocus.util.daterangedsl;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/**
 * Evaluates a range spec to create a java.util.Calendar
 * @author Risk Focus Inc.
 */
public class DateRangeDslDateEvaluator {
	final static String SECONDS = "seconds", MINUTES = "minutes",
			HOURS = "hours", DAYS = "days", WEEKS = "weeks", MONTHS = "months",
			YEARS = "years";
	private final SimpleDateFormat dateFormatter = new SimpleDateFormat("yyyymmdd");

	protected static final Map<String, Integer> periodToDateFieldMapping = createPeriodToDateFieldMapping();
	protected static final Set<String> timeVariantPeriods = createTimeVariantPeriodSet(); 
	protected boolean from;
	protected boolean relative;
	protected int dateOrOffset;
	protected boolean relativeToNow;
	protected String period;
	protected boolean now;

	public DateRangeDslDateEvaluator(boolean from) {
		super();
		this.from = from;
	}

	public boolean isRelative() {
		return relative;
	}

	public void setRelative(boolean relative) {
		this.relative = relative;
	}

	public boolean isNow() {
		return now;
	}

	public void setNow(boolean now) {
		this.now = now;
	}

	public int getDateOrOffset() {
		return dateOrOffset;
	}

	public void setDateOrOffset(int dateOrOffset) {
		this.dateOrOffset = dateOrOffset;
	}

	public boolean isRelativeToNow() {
		return relativeToNow;
	}

	public void setRelativeToNow(boolean relativeToNow) {
		this.relativeToNow = relativeToNow;
	}

	public String getPeriod() {
		return period;
	}

	public void setPeriod(String period) {
		this.period = period;
	}

	public Calendar evaluate(Calendar relativeTo) throws Exception {
		Calendar calendar = Calendar.getInstance();

		if (isNow())
			return calendar;

		if (relative) {
			calendar.setTimeInMillis(relativeTo.getTimeInMillis());
			calendar.add(periodToDateFieldMapping.get(period), from ? -dateOrOffset
					: dateOrOffset);
			if (timeVariantPeriods.contains(period)) {
				setTime(calendar, relativeTo);
			} else if (relativeToNow) {
				throw new Exception("Cannot adjust time for [" + period + "]");
			}
		} else {
			calendar.setTime(dateFormatter.parse(Integer.toString(dateOrOffset)));
			setTime(calendar, relativeTo);
		}
		return calendar;
	}

	protected void setTime(Calendar calendar, Calendar relativeTo) {
		if (relativeToNow) {
			setTimeRelativeTo(calendar, relativeTo);
		} else {
			resetTime(calendar);
		}
	}

	protected void resetTime(Calendar calendar) {
		calendar.set(Calendar.HOUR_OF_DAY, 0);
		calendar.set(Calendar.MINUTE, 0);
		calendar.set(Calendar.SECOND, 0);
		calendar.set(Calendar.MILLISECOND, 0);
	}

	protected void setTimeRelativeTo(Calendar calendar, Calendar relativeTo) {
		calendar.set(Calendar.HOUR_OF_DAY, relativeTo.get(Calendar.HOUR_OF_DAY));
		calendar.set(Calendar.MINUTE, relativeTo.get(Calendar.MINUTE));
		calendar.set(Calendar.SECOND, relativeTo.get(Calendar.SECOND));
		calendar.set(Calendar.MILLISECOND, relativeTo.get(Calendar.MILLISECOND));
	}

	protected static Set<String> createTimeVariantPeriodSet() 
	{
		Set<String> set = new HashSet<String>();
		set.add(YEARS);
		set.add(MONTHS);
		set.add(WEEKS);
		set.add(DAYS);
		return set;
	}

	protected static Map<String, Integer> createPeriodToDateFieldMapping() 
	{
		Map<String,Integer> map = new HashMap<String, Integer>();
		map.put(SECONDS, Calendar.SECOND);
		map.put(MINUTES, Calendar.MINUTE);
		map.put(HOURS, Calendar.HOUR);
		map.put(DAYS, Calendar.DAY_OF_YEAR);
		map.put(WEEKS, Calendar.WEEK_OF_YEAR);
		map.put(MONTHS, Calendar.MONTH);
		map.put(YEARS, Calendar.YEAR);
		return map;
	}
}
