/*
 * Copyright [2011] [Risk Focus Inc]
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package com.riskfocus.util.daterangedsl;

import java.io.StringReader;
import java.text.DateFormat;
import java.util.Calendar;
import java.util.Iterator;
import java.util.List;

import org.antlr.runtime.ANTLRReaderStream;
import org.antlr.runtime.TokenRewriteStream;
import org.antlr.runtime.tree.CommonTree;

import com.riskfocus.util.daterangedsl.parser.DateRangeDslLexer;
import com.riskfocus.util.daterangedsl.parser.DateRangeDslParser;

/**
 * Utility to convert a sentence in the date range DSL to a date range [from, to]
 * @author Risk Focus Inc.
 * @since 2011
 */
public class DateRangeDslUtil {

	private static final String RANGEEND = "RANGEEND";
	private static final String RELATIVE = "RELATIVE";
	private static final String RELATIVETONOW = "/thistime";
	private static final String ABSOLUTE = "ABSOLUTE";
	private static final String NOW = "now";
	private static final String DAYS = "days";
	private static final String YESTERDAY = "yesterday";
	private static final String RANGESTART = "RANGESTART";

	public static void main(String[] args) throws Exception {
		DateRangeDslUtil util = new DateRangeDslUtil();
		DateRange dateRange = util.getDateRange(args[0]);
		DateFormat df = DateFormat.getDateTimeInstance();
		System.out.println("from: " + df.format(dateRange.getFrom().getTime()));
		System.out.println("to: " + df.format(dateRange.getTo().getTime()));
	}
	
	protected DateRange getDateRange(String rangeSpec) throws Exception
	{
		DateRangeDslLexer lex = new DateRangeDslLexer(new ANTLRReaderStream(
				new StringReader(rangeSpec))) {
			@Override
			public void emitErrorMessage(String msg) {
				throw new RuntimeException("Syntax error: " + msg);
			}
		};

		TokenRewriteStream tokens = new TokenRewriteStream(lex);
		DateRangeDslParser parser = new DateRangeDslParser(tokens);
		DateRangeDslParser.dateRangeModel_return drm = parser.dateRangeModel();

		DateRangeDslDateEvaluator fromEvaluator = new DateRangeDslDateEvaluator(
				true);
		DateRangeDslDateEvaluator toEvaluator = new DateRangeDslDateEvaluator(
				false);

		CommonTree tree = (CommonTree) drm.getTree();

		System.out.println(tree.toStringTree());

		@SuppressWarnings("unchecked")
		List<CommonTree> children = tree.getChildren();
		DateRangeDslDateEvaluator currentEvaluator = fromEvaluator;

		Iterator<CommonTree> iter = children.iterator();
		while (iter.hasNext()) {
			String text = iter.next().getText();
			if (RANGESTART.equals(text)) {
				currentEvaluator = fromEvaluator;
			} else if (RANGEEND.equals(text)) {
				currentEvaluator = toEvaluator;
			} else if (ABSOLUTE.equals(text)) {
				String qtyStr = iter.next().getText();
				Integer qty = Integer.parseInt(qtyStr);
				currentEvaluator.setDateOrOffset(qty);
				currentEvaluator.setDateOrOffset(qty);
			} else if (RELATIVE.equals(text)) {
				currentEvaluator.setRelative(true);
				String qtyStr = iter.next().getText();

				if (YESTERDAY.equals(qtyStr)) {
					currentEvaluator.setDateOrOffset(1);
					currentEvaluator.setPeriod(DAYS);
				} else {
					Integer qty = Integer.parseInt(qtyStr);
					currentEvaluator.setDateOrOffset(qty);
					String period = iter.next().getText();
					currentEvaluator.setPeriod(period);
				}
			} else if (RELATIVETONOW.equals(text)) {
				currentEvaluator.setRelativeToNow(true);
			} else if (NOW.equals(text)) {
				currentEvaluator.setNow(true);
			} else {
				throw new Exception("Unexpected token:" + text);
			}

		}
		Calendar now = Calendar.getInstance();

		Calendar from = fromEvaluator.evaluate(now);
		Calendar to = toEvaluator.evaluate(from);

		return new DateRange(from, to);
	}
}
